--------------------------------------------------------------------------------
-- Weapon HL2 Crossbow by Vectar666
-- Based on:
-- Weapon Crossbow + Projectile Bolt
-- Original Carnage Contest Weapon
-- Script by DC, August 2009, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
if cc.hl2==nil then cc.hl2={} end
cc.hl2.crossbow={}
cc.hl2.crossbow.bolt={}

-- Load & Prepare Ressources
cc.hl2.crossbow.gfx_wpn=loadgfx("weapons/HL2_crossbow.png")				-- Weapon Image
setmidhandle(cc.hl2.crossbow.gfx_wpn)
cc.hl2.crossbow.gfx_pro=loadgfx("weapons/HL2_crossbow_bolt.bmp")					-- Projectile Image
setmidhandle(cc.hl2.crossbow.gfx_pro)
cc.hl2.crossbow.sfx_attack=loadsfx("HL2_xbow.wav")				-- Attack Sound
cc.hl2.crossbow.sfx_impact=loadsfx("HL2_xbow_bolt.wav")				-- Impact Sound

--------------------------------------------------------------------------------
-- Weapon: Crossbow
--------------------------------------------------------------------------------

cc.hl2.crossbow.id=addweapon("cc.hl2.crossbow","Crossbow",cc.hl2.crossbow.gfx_wpn,1,2)	-- Add Weapon (1 use)
cc.hl2.crossbow.ammo=1											-- 1 bolt

function cc.hl2.crossbow.draw()										-- Draw
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(0.6,0.6)
	drawinhand(cc.hl2.crossbow.gfx_wpn,6,0)
	-- HUD ammobar
	if cc.hl2.crossbow.ammo-weapon_shots>0 then
		hudammobar(cc.hl2.crossbow.ammo-weapon_shots,cc.hl2.crossbow.ammo)
	end
	-- HUD Crosshair
	if cc.hl2.crossbow.ammo-weapon_shots>0 then
		hudcrosshair(7,3)
	end
end

function cc.hl2.crossbow.attack(attack)								-- Attack
	-- Attack
	if weapon_shots==0 and attack==1 then
		-- No more weapon switching!
		useweapon(0)
		-- Attack
		playsound(cc.hl2.crossbow.sfx_attack)
		weapon_shots=1
		id=createprojectile(cc.hl2.crossbow.bolt.id)
		projectiles[id]={}
		-- Ignore collision with current player at beginning
		projectiles[id].ignore=playercurrent()
		-- Set initial position of projectile
		projectiles[id].x=getplayerx(0)+(7*getplayerdirection(0))-math.sin(math.rad(getplayerrotation(0)))*5.0
		projectiles[id].y=getplayery(0)+3+math.cos(math.rad(getplayerrotation(0)))*5.0
		-- Set speed of projectile
		projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*14.0
		projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*14.0
		-- Initial movement
		projectiles[id].x=projectiles[id].x-projectiles[id].sx*0.3
		projectiles[id].y=projectiles[id].y-projectiles[id].sy*0.3
		-- Effects
		recoil(4)
		-- End Turn
		endturn()
	end
end

--------------------------------------------------------------------------------
-- Projectile: Bolt
--------------------------------------------------------------------------------

cc.hl2.crossbow.bolt.id=addprojectile("cc.hl2.crossbow.bolt")		-- Add Projectile

function cc.hl2.crossbow.bolt.draw(id)							-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(cc.hl2.crossbow.gfx_pro,projectiles[id].x,projectiles[id].y)
	-- Draw arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.hl2.crossbow.bolt.update(id)						-- Update
	-- Wind + Gravity influence on speed
	projectiles[id].sx=projectiles[id].sx+getwind()*0.05
	projectiles[id].sy=projectiles[id].sy+getgravity()*0.5
	-- Move
	cc.hl2.crossbow.bolt.move(id)
end

function cc.hl2.crossbow.bolt.move(id)
	rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/2)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby		
		-- Collision
		if collision(col2x2,projectiles[id].x+math.sin(math.rad(rot))*4,projectiles[id].y-math.cos(math.rad(rot))*4,1,1)==1 then
			if terraincollision()==1 or playercollision()~=projectiles[id].ignore then
				if playercollision()~=0 then
					-- Cause damage
					playerpush(playercollision(),projectiles[id].sx/10.0,projectiles[id].sy/10.0)
					playerdamage(playercollision(),100)
					blood(projectiles[id].x+math.sin(math.rad(rot))*4,projectiles[id].y-math.cos(math.rad(rot))*4)
					playsound(sfx_splatter3)
				else
					-- Draw bolt in terrain
					terrainimage(cc.hl2.crossbow.gfx_pro,projectiles[id].x,projectiles[id].y,0,rot)
				end
				-- Effects
				playsound(cc.hl2.crossbow.sfx_impact)
				particle(p_smoke,projectiles[id].x+math.sin(math.rad(rot))*4,projectiles[id].y-math.cos(math.rad(rot))*4)
				particlefadealpha(0.006)
				-- Free projectile
				freeprojectile(id)
				return 1
			end
		else
			projectiles[id].ignore=0
		end
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			if math.random(1,2)==1 then
				playsound(sfx_hitwater2)
			else
				playsound(sfx_hitwater3)
			end
			-- Free projectile
			freeprojectile(id)
			return 1
		end
	end
end