--------------------------------------------------------------------------------
-- HL2 Gravity Gun (Zero-point energy manipulator) weapon
-- Based on:
-- Weapon Gamma Gun + Projectile Ray
-- Original Carnage Contest Weapon
-- Script by DC, September 2009, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
if cc.hl2==nil then cc.hl2={} end
cc.hl2.gravitygun={}
cc.hl2.gravitygun.ray={}

-- Load & Prepare Ressources
cc.hl2.gravitygun.gfx_wpn=loadgfx("weapons/HL2_Gravitygun.bmp")					-- Weapon Image
setmidhandle(cc.hl2.gravitygun.gfx_wpn)
cc.hl2.gravitygun.gfx_beam=loadgfx("weapons/HL2_Gravitygun_beam.bmp")
setmidhandle(cc.hl2.gravitygun.gfx_beam)
cc.hl2.gravitygun.sfx_impact=loadsfx("HL2_GravityGun.wav")					-- Impact Sound
cc.hl2.gravitygun.sfx_dryfire=loadsfx("HL2_GravityGun_dryfire.wav")

--------------------------------------------------------------------------------
-- Weapon: Gamma Gun
--------------------------------------------------------------------------------

cc.hl2.gravitygun.id=addweapon("cc.hl2.gravitygun","Gravity Gun",cc.hl2.gravitygun.gfx_wpn,0)	-- Add Weapon (0 uses)
cc.hl2.gravitygun.ammo=3															-- 3 Shots

function cc.hl2.gravitygun.draw()										-- Draw
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	drawinhand(cc.hl2.gravitygun.gfx_wpn,6,0)
	-- HUD ammobar
	if cc.hl2.gravitygun.ammo-weapon_shots>0 then
		hudammobar(cc.hl2.gravitygun.ammo-weapon_shots,cc.hl2.gravitygun.ammo)
	end
	-- HUD Crosshair
	if cc.hl2.gravitygun.ammo-weapon_shots>0 then
		hudcrosshair(7,3)
	end
	--if 
	--setblend(blend_alpha)
	--setscale
end

function cc.hl2.gravitygun.attack(attack)								-- Attack
	-- Decrement timer
	if weapon_timer>0 then
		weapon_timer=weapon_timer-1
	end
	--if cc.hl2.gravitygun.drawtimer==nil then cc.hl2.gravitygun.drawtimer=0 end
	-- Attack
	if weapon_shots<cc.hl2.gravitygun.ammo and weapon_timer<=0 and attack==1 then
		-- No more weapon switching!
		-- Reset Timer
		weapon_timer=25
		-- Attack
		id=createprojectile(cc.hl2.gravitygun.ray.id)
		projectiles[id]={}
		-- Ignore collision with current player at beginning
		projectiles[id].ignore=playercurrent()
		-- Set initial position of projectile
		projectiles[id].x=getplayerx(0)+(7*getplayerdirection(0))-math.sin(math.rad(getplayerrotation(0)))*5.0
		projectiles[id].y=getplayery(0)+3+math.cos(math.rad(getplayerrotation(0)))*5.0
		-- Set speed of projectile
		projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*75.0
		projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*75.0
		-- Timer
		projectiles[id].timer=0
		for i=1,3,1 do
			if cc.hl2.gravitygun.ray.move(id)==1 then
				break
			end
		end
		-- Effects
		recoil(3)
		particle(p_muzzle,getplayerx(0)+(getplayerdirection(0)*7)+math.sin(math.rad(getplayerrotation(0)))*12,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*12)
		particlecolor(255,255,0)
		particlefadealpha(0.01)
		particle(p_smoke,getplayerx(0)+(getplayerdirection(0)*7)+math.sin(math.rad(getplayerrotation(0)))*12,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*12)
		particlespeed(-0.2+math.random()*0.4+getwind()*10.0,-1.0+math.random()*0.6)
		particlefadealpha(0.005)
		-- End Turn

	end
end

--------------------------------------------------------------------------------
-- Projectile: Ray
--------------------------------------------------------------------------------

cc.hl2.gravitygun.ray.id=addprojectile("cc.hl2.gravitygun.ray")			-- Add Projectile

function cc.hl2.gravitygun.ray.draw(id)							-- Draw
	-- Setup draw mode
	setblend(blend_light)
	setalpha(1)
	setscale(1.0,0.5)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
end

function cc.hl2.gravitygun.ray.update(id)							-- Update
	-- Move
	cc.hl2.gravitygun.ray.move(id)
end

function cc.hl2.gravitygun.ray.move(id)
	-- Timer (free after 10 secs)
	projectiles[id].timer=projectiles[id].timer+1
	if projectiles[id].timer>20 then
		-- Free projectile
		freeprojectile(id)
		playsound(cc.hl2.gravitygun.sfx_dryfire)
		return 1
	end
	rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy)))
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby		
		-- Collision
		if collision(col3x3,projectiles[id].x+math.sin(math.rad(rot))*5,projectiles[id].y-math.cos(math.rad(rot))*5,0,1)==1 then
			if playercollision()~=projectiles[id].ignore then
				useweapon(0)
				weapon_shots=weapon_shots+1
				-- Push
				playerpush(playercollision(),projectiles[id].sx/8,projectiles[id].sy/8)
				-- Effects
				setblend(blend_light)
				setalpha(1)
				setscale(1.0,math.sqrt((getplayerx(0)-projectiles[id].x)^2+(getplayery(0)-projectiles[id].y)^2)/24)
				setrotation(rot)
				drawimage(cc.hl2.gravitygun.gfx_beam,getplayerx(0),getplayery(0))
				playsound(cc.hl2.gravitygun.sfx_impact)
				particle(p_smoke,projectiles[id].x+math.sin(math.rad(rot))*5,projectiles[id].y-math.cos(math.rad(rot))*5)
				particlefadealpha(0.006)
				particle(p_muzzle,projectiles[id].x+math.sin(math.rad(rot))*5,projectiles[id].y-math.cos(math.rad(rot))*5)
				particlesize(1.5,1.5)
				particlecolor(255,100,0)
				particlefadealpha(0.01)
				for j=1,20,1 do
					particle(p_lightpuff,projectiles[id].x+math.sin(math.rad(rot))*5,projectiles[id].y-math.cos(math.rad(rot))*5)
					particlespeed(math.random(-20,20)*0.1,math.random(-20,20)*0.1)
					particlefadealpha(0.04)
					particlesize(math.random(40,70)*0.1,math.random(40,70)*0.1)
					particlecolor(255,100,0)
				end
				-- Free projectile
				freeprojectile(id)
				if (weapon_shots>=cc.hl2.gravitygun.ammo) then
					endturn()
				end
				return 1
			end
		else
			projectiles[id].ignore=0
		end
		if collision(col3x3,projectiles[id].x+math.sin(math.rad(rot))*5,projectiles[id].y-math.cos(math.rad(rot))*5,1,0)==1 then
			freeprojectile(id)
			playsound(cc.hl2.gravitygun.sfx_dryfire)
			return 1
		end
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_smoke,projectiles[id].x,projectiles[id].y)
			-- Free projectile
			freeprojectile(id)
			playsound(cc.hl2.gravitygun.sfx_dryfire)
			return 1
		end
	end
end