--------------------------------------------------------------------------------
-- Half-life 2 Combine Impulse Rifle weapon by Vectar666
-- Based on:
-- Weapon Pistol + Projectile Bullet
-- and
-- Weapon Plasma Gun + Projectile Ray
-- Original Carnage Contest Weapons
-- Script by DC, August 2009, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
if cc.hl2==nil then cc.hl2={} end
cc.hl2.IR={}
cc.hl2.IR.bullet={}
cc.hl2.IR.ball={}

-- Load & Prepare Ressources
cc.hl2.IR.gfx_wpn=loadgfx("weapons/HL2_ImpulseRifle.bmp")					-- Weapon Image
setmidhandle(cc.hl2.IR.gfx_wpn)
cc.hl2.IR.gfx_pro=loadgfx("weapons/HL2_ImpulseTracer.png")					-- Projectile Image
setmidhandle(cc.hl2.IR.gfx_pro)
cc.hl2.IR.gfx_ball=loadgfx("weapons/HL2_Combineball.bmp")
setmidhandle(cc.hl2.IR.gfx_ball)
cc.hl2.IR.sfx_attack=loadsfx("HL2_IR.wav")						-- Attack Sound
cc.hl2.IR.sfx_ball=loadsfx("HL2_IR_altfire.wav")
cc.hl2.IR.sfx_impact=loadsfx("HL2_ball_explode.wav")
cc.hl2.IR.sfx_bounce=loadsfx("HL2_ball_bounce.wav")
cc.hl2.IR.sfx_kill=loadsfx("HL2_disintegrate.wav")

--------------------------------------------------------------------------------
-- Weapon: Pistol
--------------------------------------------------------------------------------

cc.hl2.IR.id=addweapon("cc.hl2.IR","Impulse Rifle",cc.hl2.IR.gfx_wpn,1,2)	-- Add Weapon
cc.hl2.IR.ammo=30												-- 45 Bullets

function cc.hl2.IR.draw()										-- Draw
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	drawinhand(cc.hl2.IR.gfx_wpn,6,0)
	-- HUD ammobar
	if cc.hl2.IR.ammo-weapon_shots>0 then
		if weapon_timer==1 then
		hudammobar(cc.hl2.IR.ammo-weapon_shots,cc.hl2.IR.ammo)
		else
		hudammobar(1-weapon_shots,1)
		end
	end
	-- HUD Crosshair
	if cc.hl2.IR.ammo-weapon_shots>0 then
		hudcrosshair(7,3)
		hudtimer(1,1,2)
	end
end

function cc.hl2.IR.attack(attack)								-- Attack
	-- Decrement timer
	if cc.hl2.IR.weapon_timer==nil then cc.hl2.IR.weapon_timer=0 end
	if cc.hl2.IR.weapon_timer>0 then
		cc.hl2.IR.weapon_timer=cc.hl2.IR.weapon_timer-1
	end
	-- Start burst
	if attack==1 then
		-- No more weapon switching!
		useweapon(0)
		weapon_mode=weapon_timer
	end
	-- Attack
	if weapon_shots<cc.hl2.IR.ammo and cc.hl2.IR.weapon_timer<=0 and weapon_mode==1 then
		-- Reset Timer
		cc.hl2.IR.weapon_timer=5
		-- Attack
		playsound(cc.hl2.IR.sfx_attack)
		weapon_shots=weapon_shots+1
		id=createprojectile(cc.hl2.IR.bullet.id)
		projectiles[id]={}
		-- Ignore collision with current player at beginning
		projectiles[id].ignore=playercurrent()
		-- Set initial position of projectile
		projectiles[id].x=getplayerx(0)+(7*getplayerdirection(0))-math.sin(math.rad(getplayerrotation(0)))*5.0
		projectiles[id].y=getplayery(0)+3+math.cos(math.rad(getplayerrotation(0)))*5.0
		-- Set speed of projectile
		math.randomseed(weapon_shots*12345)
		offset=-1.5+math.random()*3.0
		projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)+offset))*20.0
		projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)+offset))*20.0
		
		projectiles[id].timer=0
		-- Initial movement
		projectiles[id].x=projectiles[id].x-projectiles[id].sx*1
		projectiles[id].y=projectiles[id].y-projectiles[id].sy*1
		for i=1,3,1 do
			if cc.hl2.IR.bullet.move(id)==1 then
				break
			end
		end
		-- Effects
		recoil(3)
		particle(p_muzzle,getplayerx(0)+(getplayerdirection(0)*7)+math.sin(math.rad(getplayerrotation(0)))*16,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*16)
		particle(p_smoke,getplayerx(0)+(getplayerdirection(0)*7)+math.sin(math.rad(getplayerrotation(0)))*16,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*16)
		particlespeed(-0.2+math.random()*0.4+getwind()*10.0,-1.0+math.random()*0.6)
		particlefadealpha(0.005)
		-- End Turn
		endturn()
	end
	if	attack==1 and weapon_mode==2 and weapon_shots==0 then
		cc.hl2.IR.weapon_timer=80
		weapon_shots=weapon_shots+1
		playsound(cc.hl2.IR.sfx_ball)
		id=createprojectile(cc.hl2.IR.ball.id)
		projectiles[id]={}
		-- Ignore collision with current player at beginning
		projectiles[id].ignore=playercurrent()
		-- Set initial position of projectile
		projectiles[id].x=getplayerx(0)+(6*getplayerdirection(0))+math.sin(math.rad(getplayerrotation(0)))*10.0
		projectiles[id].y=getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*10.0
		-- Initial movement
		projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*13
		projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*13
		projectiles[id].x=projectiles[id].x-projectiles[id].sx
		projectiles[id].y=projectiles[id].y-projectiles[id].sy
		projectiles[id].timer=0
		cc.hl2.IR.ball.move(id)
		-- Effects
		recoil(5)
		-- End Turn
		endturn()
	end
end

--------------------------------------------------------------------------------
-- Projectile: Bullet
--------------------------------------------------------------------------------

cc.hl2.IR.bullet.id=addprojectile("cc.hl2.IR.bullet")		-- Add Projectile

function cc.hl2.IR.bullet.draw(id)							-- Draw
	-- Setup draw mode
	setblend(blend_light)
	setalpha(1)
	setcolor(255,255,255)
	setscale(0.8,0.8)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(cc.hl2.IR.gfx_pro,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.hl2.IR.bullet.update(id)						-- Update
	-- Wind + Gravity influence on speed
	projectiles[id].sx=projectiles[id].sx+getwind()*0.01
	projectiles[id].sy=projectiles[id].sy+getgravity()*0.1
	-- Move
	cc.hl2.IR.bullet.move(id)
end

function cc.hl2.IR.bullet.move(id)
	-- Timer (free after 10 secs)
	projectiles[id].timer=projectiles[id].timer+1
	if projectiles[id].timer>500 then
		-- Free projectile
		freeprojectile(id)
		return 1
	end
	rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby		
		-- Collision
		if collision(col3x3,projectiles[id].x+math.sin(math.rad(rot))*20,projectiles[id].y-math.cos(math.rad(rot))*20,1,1)==1 then
			if terraincollision()==1 or playercollision()~=projectiles[id].ignore then
				-- Cause damage
				if playercollision()~=0 and playercollision()~=playercurrent() then
					playerpush(playercollision(),projectiles[id].sx/10.0,projectiles[id].sy/10.0)
					playerdamage(playercollision(),8)
					blood(projectiles[id].x+math.sin(math.rad(rot))*20,projectiles[id].y-math.cos(math.rad(rot))*20)
				end
				-- Destroy terrain
				for j=20,22,1 do
					terraincircle(projectiles[id].x+math.sin(math.rad(rot))*j,projectiles[id].y-math.cos(math.rad(rot))*j,3,0x00000000)
				end
				-- Effects
				playsound(sfx_ricochet1)
				particle(p_smoke,projectiles[id].x+math.sin(math.rad(rot))*21,projectiles[id].y-math.cos(math.rad(rot))*21)
				particlefadealpha(0.006)
				particle(p_muzzle,projectiles[id].x+math.sin(math.rad(rot))*21,projectiles[id].y-math.cos(math.rad(rot))*21)
				-- Free projectile
				freeprojectile(id)
				return 1
			end
		else
			projectiles[id].ignore=0
		end
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			if math.random(1,2)==1 then
				playsound(sfx_hitwater2)
			else
				playsound(sfx_hitwater3)
			end
			-- Free projectile
			freeprojectile(id)
			return 1
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: ball
--------------------------------------------------------------------------------

cc.hl2.IR.ball.id=addprojectile("cc.hl2.IR.ball")		-- Add Projectile

function cc.hl2.IR.ball.draw(id)								-- Draw
	-- Setup draw mode
	setblend(blend_light)
	setalpha(1)
	setcolor(255,255,255)
	setscale(0.7,0.7)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(cc.hl2.IR.gfx_ball,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.hl2.IR.ball.update(id)							-- Update
	-- Move
	cc.hl2.IR.ball.move(id)
end

function cc.hl2.IR.ball.move(id)
	-- Timer (free after 5 secs)
	projectiles[id].timer=projectiles[id].timer+1
	if projectiles[id].timer>250 then
		-- Free projectile
		particle(p_muzzle,projectiles[id].x,projectiles[id].y)
		particlecolor(155,255,255)
		for angle=0,350,5 do
			particle(p_smoke,projectiles[id].x,projectiles[id].y)
			particlespeed(math.sin(math.rad(angle))*1.8,-math.cos(math.rad(angle))*1.5)
			particlecolor(155,155,255)
		end
		freeprojectile(id)
		playsound(cc.hl2.IR.sfx_impact)
		return 1
	end
	rot=math.atan2(projectiles[id].sx,-projectiles[id].sy)
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/5)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby		
		-- Collision
		if collision(col3x3,projectiles[id].x+math.sin(rot)*5,projectiles[id].y-math.cos(rot)*5,1,1)==1 then
			if terraincollision()==1 and playercollision()==0 then
				-- define angle
				local mx=0
				local my=0
				for a=0,350,10 do
					local ix=math.cos(math.rad(a))*10
					local iy=math.sin(math.rad(a))*10
					if collision(col2x2,projectiles[id].x+ix,projectiles[id].y+iy,1,0)==1 then
						mx=mx+ix
						my=my+iy
					end
				end
				local an=math.atan2(mx,my)
				local ai=math.atan2(projectiles[id].sx,projectiles[id].sy)
				local ao=an*2-ai+math.pi
				local s=math.sqrt(projectiles[id].sx^2+projectiles[id].sy^2)
				projectiles[id].sx=math.sin(ao)*s
				projectiles[id].sy=math.cos(ao)*s
					msubx=projectiles[id].sx/msubt
					msuby=projectiles[id].sy/msubt
					-- Effects
					playsound(cc.hl2.IR.sfx_bounce)
					particle(p_smoke,projectiles[id].x+math.sin(rot)*5,projectiles[id].y-math.cos(math.rad(rot))*5)
					particlefadealpha(0.006)
					particle(p_muzzle,projectiles[id].x+math.sin(rot)*5,projectiles[id].y-math.cos(math.rad(rot))*5)
					particlesize(1.5,1.5)
					particlecolor(200,200,255)
					particlefadealpha(0.01)
					for j=1,3,1 do
						particle(p_lightpuff,projectiles[id].x+math.sin(rot)*5,projectiles[id].y-math.cos(math.rad(rot))*5)
						particlespeed(math.random(-20,20)*0.1,math.random(-20,20)*0.1)
						particlefadealpha(0.04)
						particlesize(math.random(40,70)*0.1,math.random(40,70)*0.1)
						particlecolor(200,200,255)
					end
				--end
			elseif playercollision()~=projectiles[id].ignore then
				playerdamage(playercollision(),1000)
					for j=1,10,1 do
						particle(p_smoke,projectiles[id].x+math.sin(rot)*5,projectiles[id].y-math.cos(math.rad(rot))*5)
						particlespeed(math.random(-20,20)*0.1,math.random(-20,20)*0.1)
						particlefadealpha(0.005)
						particlesize(math.random(40,70)*0.5,math.random(40,70)*0.5)
						particlecolor(30,20,50)
					end
				playsound(cc.hl2.IR.sfx_kill)
				-- define angle
				local mx=0
				local my=0
				for a=0,350,10 do
					local ix=math.cos(math.rad(a))*10
					local iy=math.sin(math.rad(a))*10
					if collision(col2x2,projectiles[id].x+ix,projectiles[id].y+iy,1,1)==1 then
						mx=mx+ix
						my=my+iy
					end
				end
				local an=math.atan2(mx,my)
				local ai=math.atan2(projectiles[id].sx,projectiles[id].sy)
				local ao=an*2-ai+math.pi
				local s=math.sqrt(projectiles[id].sx^2+projectiles[id].sy^2)
				projectiles[id].sx=math.sin(ao)*s
				projectiles[id].sy=math.cos(ao)*s
			end
		else
			projectiles[id].ignore=0
		end
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			playsound(cc.hl2.IR.sfx_impact)
			particle(p_smoke,projectiles[id].x,projectiles[id].y)
			-- Free projectile
			freeprojectile(id)
			return 1
		end
	end
	scroll(projectiles[id].x,projectiles[id].y)
end